@@ -3,12 +3,7 @@
 
 #include "vctrs-core.h"
 
-void vec_assert(r_obj* x,
-                r_ssize size,
-                struct vctrs_arg* arg,
-                struct r_lazy call);
-
-void vec_check_vector(r_obj* x,
+void obj_check_vector(r_obj* x,
                       struct vctrs_arg* arg,
                       struct r_lazy call);
 
@@ -17,7 +12,7 @@ void vec_check_size(r_obj* x,
                     struct vctrs_arg* arg,
                     struct r_lazy call);
 
-void vec_check_list(r_obj* x,
+void obj_check_list(r_obj* x,
                     struct vctrs_arg* arg,
                     struct r_lazy call);
 
